/******************************************************************/
/* make vector consistent                                         */
/******************************************************************/
#include <stdlib.h>
#include "mpi.h"

void  MakeVectorConsistent(int left_neighbour,int right_neighbour,
                           int lower_neighbour,int upper_neighbour,
                           int x_int_loc,int  y_int_loc, double *vec)
{
   int i, nx, ny, length;
   double *sbuf, *rbuf0, *rbuf1;
   MPI_Status status;

   nx = x_int_loc+1;
   ny = y_int_loc+1;
   
   /* compute maximum of nx and ny */
   if (nx>ny)
      length = nx;
   else
      length = ny;

   /* allocate buffers for sending and receiving data */
   if (!(sbuf=malloc(length*sizeof(*sbuf))))
      MPI_Abort(MPI_COMM_WORLD, 1);
   if (!(rbuf0=malloc(length*sizeof(*rbuf0))))
      MPI_Abort(MPI_COMM_WORLD, 1);
   if (!(rbuf1=malloc(length*sizeof(*rbuf1))))
      MPI_Abort(MPI_COMM_WORLD, 1);
  
   /****************************************************************/
   /* lower and upper boundaries                                   */
   /****************************************************************/

   /* to lower neighbour */
   if ((lower_neighbour>-1))
   {
      for (i=0;i<nx;i++)
         sbuf[i] = vec[i];
      MPI_Send(sbuf, length, MPI_DOUBLE, lower_neighbour, 0,  MPI_COMM_WORLD);
   }
    /* from upper  neighbour */
   if ((upper_neighbour>-1))
      MPI_Recv(rbuf0, length, MPI_DOUBLE, upper_neighbour, 0,  MPI_COMM_WORLD, &status);
   
   /* to  upper neighbour */
   if ((upper_neighbour>-1))
   {
       for (i=0;i<nx;i++)
         sbuf[i] = vec[nx*(ny-1)+i];
       MPI_Send(sbuf, length, MPI_DOUBLE, upper_neighbour, 0,  MPI_COMM_WORLD);     
   }
   /* from lower neighbour */
   if ((lower_neighbour>-1))
      MPI_Recv(rbuf1, length, MPI_DOUBLE, lower_neighbour, 0,  MPI_COMM_WORLD, &status);
 
   /* update values */
   if ((upper_neighbour>-1))
   {
      for (i=0;i<nx;i++)
         vec[nx*(ny-1)+i] += rbuf0[i];
   }

   if ((lower_neighbour>-1))
   {
      for (i=0;i<nx;i++)
         vec[i] += rbuf1[i];
   }

   /****************************************************************/
   /* left and right boundaries                                    */
   /****************************************************************/

  /* to left neighbour */
   if ((left_neighbour>-1))
   {
       for (i=0;i<ny;i++)
         sbuf[i] = vec[i*nx];
       MPI_Send(sbuf, length, MPI_DOUBLE, left_neighbour, 0,  MPI_COMM_WORLD);     
   }
   /* from right neighbour */
   if ((right_neighbour>-1))
      MPI_Recv(rbuf0, length, MPI_DOUBLE, right_neighbour, 0,  MPI_COMM_WORLD, &status);
      
   /* to right neighbour */
   if ((right_neighbour>-1))
   {
       for (i=0;i<ny;i++)
         sbuf[i] = vec[(i+1)*nx-1];
       MPI_Send(sbuf, length, MPI_DOUBLE, right_neighbour, 0,  MPI_COMM_WORLD);     
   }
   /* from left neighbour */
   if ((left_neighbour>-1))
      MPI_Recv(rbuf1, length, MPI_DOUBLE, left_neighbour, 0,  MPI_COMM_WORLD, &status);
      
   /* update values */
   if ((right_neighbour>-1))
   {
      for (i=0;i<ny;i++)
         vec[(i+1)*nx-1] += rbuf0[i];
   }
   if ((left_neighbour>-1))
   {
      for (i=0;i<ny;i++)
         vec[i*nx] += rbuf1[i];
   }
  

   free(sbuf);
   free(rbuf0);

   return;
}
