#include "mpi.h"
#include "headers.h"

#include <stdlib.h>
#include <string.h>
#include <math.h>
/**************************************************************/
/* jacobi iteration                                           */
/**************************************************************/
void jacobi(int rank,
	    int left_neighbour,int right_neighbour,
	    int lower_neighbour,int upper_neighbour,
	    int x_int_loc,int  y_int_loc,
	    int maxit, double eps, double damp,
	    int dof_loc, int *KCol, int *RowPtr,
	    double *matrix_entries, double *sol, double *rhs,
	    double *defect)
{
  int i, j, k, index, start, end;
  double *r, *diag, residual, residual_cons, old_residual;
  

  r = malloc(dof_loc*sizeof(double));
  diag = malloc(dof_loc*sizeof(double));
  old_residual = 1;

  /* compute diagonal of matrix */
  /* loop over all rows of local matrix */
  for(i=0;i<dof_loc;i++)
  {
    start = RowPtr[i];
    end = RowPtr[i+1];
    /* loop over all columns belonging to this row */
    for(j=start;j<end;j++)
    {
       /* index of column */
      index = KCol[j];
      /* if the same as row index */
      if (index==i)
      {
         /* diagonal found */
         diag[i] = matrix_entries[j];
	break;
      }
    }
  }  

  /* make diag consistently */
  MakeVectorConsistent(left_neighbour, right_neighbour,
                       lower_neighbour, upper_neighbour,
                       x_int_loc, y_int_loc, diag);
  
 
  for (k=0;k<maxit;k++)
  {
    /* compute defect, stored inconsistently */
    MatVecDefect(dof_loc, RowPtr, KCol, matrix_entries, sol, 
		 rhs, defect);
    /* copy defect to r */
    memcpy(r,defect,dof_loc*sizeof(double));

    /* make defect consistent */
    MakeVectorConsistent(left_neighbour, right_neighbour,
			 lower_neighbour, upper_neighbour,
			 x_int_loc, y_int_loc, defect);
    /* compute inner product of r and defect, residual is stored
       inconsistently */
    residual = 0;
    for (i=0;i<dof_loc;i++)
       residual +=  defect[i] * r[i];

    /* make residual consistent */
    MPI_Allreduce(&residual, &residual_cons, 1, MPI_DOUBLE, MPI_SUM, 
		  MPI_COMM_WORLD);
    residual_cons= sqrt(residual_cons);
    
    /*if (rank==0)
      printf("ite %3d %8.6e rate %g\n",k, residual_cons, 
	     residual_cons/old_residual);
    */
    old_residual = residual_cons;
    /* stopping criterion fulfilled, stop iteration */
    if (residual_cons<eps)
      break;
    /* else compute update */
    for (i=0;i<dof_loc;i++)
    {
       /* printf("%g %g %g %g\n ",sol[i],defect[i], diag[i],damp);*/
      sol[i] += damp*defect[i]/diag[i];
    }
       
  }
  
  if (rank==0)
    printf("Jacobi: iterations %d residual %g\n", k, residual_cons);

  free(r);
  free(diag);

  return;
}
